# encoding: UTF-8
require 'cgi'
require 'open-uri'
require 'json'

BASE_URL = "http://dev.virtualearth.net/REST/V1/Routes?"
BING_MAPS_KEY = ENV['BING_MAPS_KEY']

class BingRoute

  @@default_options = {
    :optmz => :timeWithTraffic,
    :du => :mi,
    :ra => :routeSummariesOnly,
    :key => BING_MAPS_KEY,
  }

  def initialize(origin, destination, opts=@@default_options)
    waypoints = "wp.0=#{origin.to_query}&wp.1=#{destination.to_query}"
    url = "#{BASE_URL}#{waypoints}&#{opts.to_query_string}"

    begin
      doc = open(url)
    rescue OpenURI::HTTPError => e
      puts "Whoops we got #{e.message}"
      @status = e.message
      return
    end

    directions = JSON.parse(doc.read)
    @status = directions["statusDescription"]
    @distance = directions["resourceSets"][0]["resources"][0]["travelDistance"]
    @time_in_seconds = directions["resourceSets"][0]["resources"][0]["travelDuration"]
    @traffic_time_in_seconds = directions["resourceSets"][0]["resources"][0]["travelDurationTraffic"]
  end

  def status
    @status
  end

  def distance
    return 0 if @status != "OK"
    @distance.round
  end

  def drive_time
    return 0 if @status != "OK"
    convert_to_minutes(@time_in_seconds)
  end

  def drive_time_in_traffic
    return 0 if @status != "OK"
    convert_to_minutes(@traffic_time_in_seconds)
  end

  private

  def convert_to_minutes(seconds)
    (seconds.to_f / 60).round
  end

end

class Hash

  def to_query_string
    collect do |k, v|
      "#{k}=#{CGI::escape(v.to_s)}"
    end * '&'
  end unless method_defined? :to_query_string

end

class String

  def to_query
    "#{CGI::escape(self)}"
  end

end
